/**
 * Copyright (c) 2006 Sascha Grau
 * contact: rostar@graui.de
 */

#include "multipart.h"
#include <iostream>
#include <map>

Multipart::Multipart(unsigned long n, unsigned int partnum) {

    this->n = n;
    this->partnum = partnum;

    pred = new unsigned long*[partnum];
    rank = new unsigned int*[partnum];

    for (unsigned int i=0; i<partnum; i++) {

        pred[i] = new unsigned long[n];
        rank[i] = new unsigned int[n];

        for (unsigned long j=0; j<n; j++) {
            pred[i][j] = j;
            rank[i][j] = 0;
        }
    }
}

Multipart::~Multipart() {

    for (unsigned int i=0; i<partnum; i++) {

        delete[] pred[i];
        delete[] rank[i];
    }

    delete[] pred;
    delete[] rank;
}

unsigned long Multipart::find(unsigned long elem, unsigned int part) {

    if ((part > partnum-1) || (elem > n-1)) {
        // yes, this kind of error reporting is ugly.
        return error;
    }

    unsigned long root = elem; //pred[part][elem];

    /** using bisection method to shorten find paths */
    while (pred[part][pred[part][root]] != pred[part][root]) {

        unsigned int target = root;
        root = pred[part][root];
        pred[part][target] = pred[part][root];
    }

    return pred[part][root];
}

unsigned long Multipart::unite(unsigned long elem1, 
                               unsigned long elem2,
                               unsigned int part) {

    if ((part > partnum-1) || (elem1 > n-1) || (elem2 > n-1)) {
        // yes, this kind of error reporting is ugly.
        return error;
    }

    if (elem1 == elem2)
        return elem1;

    unsigned long ret = 0;
    unsigned long root1 = find(elem1, part);
    unsigned long root2 = find(elem2, part);

    if (rank[part][root1] < rank[part][root2]) {
        pred[part][root1] = root2;
        ret = root2;
    } else {
        pred[part][root2] = root1;
        ret = root1;
        if (rank[part][root1] == rank[part][root2])
            rank[part][root1]++;
    }

    return ret;
}

void Multipart::print_part(unsigned int part) {

    if (part > partnum-1)
        return;

    std::cout << "i\t";
    for (unsigned long i=0; i<n; i++) {
        std::cout << i << "\t";
    }
    std::cout << std::endl;

    std::cout << "p(i)\t";
    for (unsigned long i=0; i<n; i++) {
        std::cout << find(i, part) << "\t";
    }
    std::cout << std::endl;
}

void Multipart::dbg_print_part(unsigned int part) {

    if (part > partnum-1)
        return;

    for (unsigned long i=0; i<n; i++) {
        std::cout << i << " - pred: " << pred[part][i];
        std::cout << " - rank: " << rank[part][i] << std::endl;
    }
}

unsigned long Multipart::part_cnt(unsigned int part) {

    std::map<unsigned long, unsigned long> s;

    if (part > partnum-1)
        return 0;

    print_part(part);

    for (unsigned long i=0; i<n; i++) {
        s[find(i, part)]++;
    }

    return s.size();
}






