// KMB 2006 July 19

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <sys/times.h>

//! useful macros for the user
#define visited(g,i) (g->v[i])
#define color(g,i)   (g->c[i])
#define cluster(g,i) (g->l[i])
#define nnodes(g)    (g->nnodes)
#define nedges(g)    (g->nedges)

//! type of a node
typedef unsigned int node_t;

struct _graph {
  int **a; // adjacency lists
  int  *v; // visited
  int  *c; // color
  int  *l; // cluster number
  unsigned int nnodes;
  unsigned int nedges;
};

//! type of a graph
typedef struct _graph* graph_t;
//! make a new empty graph with n nodes
graph_t graph_new(unsigned int n);
//! destroy a graph
void graph_clear(graph_t g);
//! set to the empty graph
void graph_empty(graph_t g);
//! add undirected edge (i,j) to the graph
void graph_add_edge(graph_t g, node_t i, node_t j);
void graph_add_node(graph_t g);
void graph_show(graph_t g);
int graph_greedy_color(graph_t g, int perm[]);
void graph_gnp(graph_t g, double p);
void graph_grg(graph_t g, double r);
void graph_grg_torus(graph_t g, double r);
int  graph_ncolors(graph_t g);
int  graph_chromatic_number(graph_t g, clock_t timeout);
int graph_clique_number(graph_t g);
int graph_check_coloring(graph_t g);
int graph_sequential_color(graph_t g, int perm[], int ub);
int graph_sequential_color_repeat(graph_t g, int n);
void graph_make_dotfile(graph_t g, char* fn);
void graph_make_dotfile_colored(graph_t g, char* fn);
void graph_to_dimacs(graph_t g, char* fn);
void graph_to_theta(graph_t g, char* fn);
int graph_node_degree(graph_t g, node_t i);
int graph_max_degree(graph_t g);
double graph_mean_degree(graph_t g);
int graph_nclusters(graph_t g);
int graph_connected(graph_t g);

struct _histogram {
  unsigned long n; // total count
  int a; // allocation 0..a-1
  unsigned int *h; // bins
  double *m; // means
  double *ss; // sums-of-squares
  char *lbl;
};

//! histogram functions
//! type of a histogram
typedef struct _histogram* histogram_t;

histogram_t histogram_new(char*);
void histogram_add(histogram_t h, int i);
void histogram_clear(histogram_t h);
//void histogram_empty(histogram_t h); // TODO
void histogram_show(histogram_t h);
void histogram_write(FILE*,histogram_t h);
int  histogram_biggest_done(histogram_t h, double eps);
int histogram_mean_done(histogram_t h, double eps);
double histogram_quantile(histogram_t h, double y);
double histogram_mean(histogram_t h);
unsigned int histogram_mode(histogram_t h);
histogram_t graph_geng_reader(FILE* f, int (graph_t), char* lbl);

//! geng functions
char* geng_getline(FILE *f);
graph_t geng_stringtograph(char *s);
int geng_graphsize(char *s);
